@@ -1,5 +1,5 @@
 package Win32::VBScript;
-$Win32::VBScript::VERSION = '0.04';
+$Win32::VBScript::VERSION = '0.05';
 use strict;
 use warnings;
 
@@ -10,16 +10,26 @@ use Win32::OLE;
 
 require Exporter;
 our @ISA         = qw(Exporter);
-our %EXPORT_TAGS = ('all' => [qw(
+our %EXPORT_TAGS = ('ini' => [qw(
     compile_prog_vbs compile_prog_js
     compile_func_vbs compile_func_js
-    cscript wscript async func flist
 )]);
 our @EXPORT      = qw();
-our @EXPORT_OK   = ( @{ $EXPORT_TAGS{'all'} } );
+our @EXPORT_OK   = ( @{ $EXPORT_TAGS{'ini'} } );
 
 my $VBRepo = $ENV{'TEMP'}.'\\Repo01';
 
+my $proxy_invoke = compile_func_vbs([ <<'EOP' ])->func('IProg');
+Function IProg(ByVal MT, ByVal MNum, ByVal MBool)
+    MBool = UCase(Mid(MBool, 1, 1))
+    Dim ZNum  : If MNum  = "1" Then ZNum  = 1    Else ZNum  = 0
+    Dim ZBool : If MBool = "T" Then ZBool = True Else ZBool = False
+
+    Dim OS : Set OS = CreateObject("WScript.Shell")
+    IProg = OS.Run(MT, ZNum, ZBool)
+End Function
+EOP
+
 sub new {
     my $pkg = shift;
 
@@ -93,9 +103,15 @@ sub new {
     write_file($file_full, $file_content);
 
     if ($type eq 'func') {
-        my $obj = Win32::OLE->GetObject('script:'.$file_full) or croak "E050: ",
-          "Couldn't Win32::OLE->GetObject('script:$file_full')",
-          " -> ".Win32::GetLastError;
+        my $obj = Win32::OLE->GetObject('script:'.$file_full);
+
+        unless ($obj) {
+            #~ my $file_text = eval{ scalar(read_file($file_full)) } || '???';
+            croak "E050: ",
+              "Couldn't Win32::OLE->GetObject('script:$file_full')",
+              " -> ".Win32::GetLastError().
+              " -> ".Win32::FormatMessage(Win32::GetLastError());
+        }
 
         for my $method (keys %dat_func) {
             $dat_func{$method} = sub { $obj->$method(@_); };
@@ -132,12 +148,20 @@ sub compile_func_js {
 
 sub _run {
     my $self = shift;
-    my ($scr, $mode) = @_;
+    my ($scr, $mode, $level) = @_;
 
     unless ($scr eq 'cscript' or $scr eq 'wscript') {
         croak "E060: Invalid script ('$scr'), expected ('cscript' or 'wscript')";
     }
 
+    unless ($mode eq 'a' or $mode eq 's') {
+        croak "E061: Invalid mode ('$mode'), expected ('a' or 's')";
+    }
+
+    unless ($level eq 'pl' or $level eq 'ms') {
+        croak "E062: Invalid level ('$level'), expected ('pl' or 'ms')";
+    }
+
     my $name = $self->{'name'};
     my $lang = $self->{'lang'};
     my $type = $self->{'type'};
@@ -159,30 +183,63 @@ sub _run {
 
     my @param = ($scr, '//Nologo', '//E:'.$engine, $full);
 
-    if ($mode eq 'a') {
-        system 1, @param; # asynchronous
+    if ($level eq 'pl') {
+        if ($mode eq 'a') {
+            system 1, @param; # asynchronous
+        }
+        elsif ($mode eq 's') {
+            system    @param; # sequentially
+        }
+        else {
+            croak "E082: Panic -- invalid mode ('$mode'), expected ('a' or 's')";
+        }
     }
-    elsif ($mode eq 's') {
-        system    @param; # sequentially
+    elsif ($level eq 'ms') {
+        my $PCmd  = join(' ', map { qq{"$_"} } @param);
+        my $PNum  = $scr eq 'cscript' ? '1' : '0';
+        my $PBool = $mode eq 's' ? 'True' : 'False';
+
+        # RC = CreateObject("WScript.Shell").Run($PCmd 0, False)
+        #   ==> 0 = CMD Prompt will not be shown,
+        #   ==> 1 = CMD Prompt will be shown,
+        #   ==> False = Do not wait for program to finish
+        #   ==> True  = Wait for program to finish
+
+        $proxy_invoke->($PCmd, $PNum, $PBool);
     }
     else {
-      croak "E082: Panic -- invalid mode ('$mode'), expected ('a' or 's')";
+        croak "E084: Panic -- invalid level ('$level'), expected ('pl' or 'ms')";
     }
 }
 
-sub cscript {
+sub pl_cscript {
     my $self = shift;
-    $self->_run('cscript', 's'); # s = sequentially
+    $self->_run('cscript', 's', 'pl'); # s = sequentially
 }
 
-sub wscript {
+sub pl_wscript {
     my $self = shift;
-    $self->_run('wscript', 's'); # s = sequentially
+    $self->_run('wscript', 's', 'pl'); # s = sequentially
 }
 
-sub async {
+sub pl_async {
     my $self = shift;
-    $self->_run('wscript', 'a'); # a = asynchronous
+    $self->_run('wscript', 'a', 'pl'); # a = asynchronous
+}
+
+sub ms_cscript {
+    my $self = shift;
+    $self->_run('cscript', 's', 'ms'); # s = sequentially
+}
+
+sub ms_wscript {
+    my $self = shift;
+    $self->_run('wscript', 's', 'ms'); # s = sequentially
+}
+
+sub ms_async {
+    my $self = shift;
+    $self->_run('wscript', 'a', 'ms'); # a = asynchronous
 }
 
 sub func {
@@ -218,24 +275,29 @@ The Win32::OLE part has been copied from Inline::WSC.
     use strict;
     use warnings;
 
-    use Win32::VBScript qw(:all);
+    use Win32::VBScript qw(:ini);
 
-    # This is the procedural interface:
-    # *********************************
+    # the cscript/wscript/async methods come
+    # in two flavours: pl_... and ms_...
+    # **************************************
 
-    my $p1 = compile_prog_js ([ qq{WScript.StdOut.WriteLine("Bonjour");} ]); cscript($p1);
-    my $p2 = compile_prog_vbs([ qq{WScript.StdOut.WriteLine "Hello"}     ]); cscript($p2);
+    compile_prog_js ([ qq{WScript.StdOut.WriteLine("Test1");} ])->pl_cscript;
+    compile_prog_js ([ qq{WScript.StdOut.WriteLine("Test2");} ])->ms_cscript;
 
-    # This is the OO interface:
-    # *************************
-
-    compile_prog_js ([ qq{WScript.StdOut.WriteLine("Test1");} ])->cscript;
-    compile_prog_vbs([ qq{WScript.StdOut.WriteLine "Test2"}   ])->cscript;
+    compile_prog_vbs([ qq{WScript.StdOut.WriteLine "Test3"}   ])->pl_cscript;
+    compile_prog_vbs([ qq{WScript.StdOut.WriteLine "Test4"}   ])->ms_cscript;
 
     # And with wscript, of course, you can use MsgBox:
     # ************************************************
 
-    compile_prog_vbs([ qq{MsgBox "Test3"} ])->wscript;
+    compile_prog_vbs([ qq{MsgBox "Test5"} ])->pl_wscript;
+    compile_prog_vbs([ qq{MsgBox "Test6"} ])->ms_wscript;
+
+    # And you can have an asynchronous MsgBox as well:
+    # ************************************************
+
+    compile_prog_vbs([ qq{MsgBox "Test7"} ])->pl_async;
+    compile_prog_vbs([ qq{MsgBox "Test8"} ])->ms_async;
 
     # You can even define functions in Visual Basic...
     # ************************************************
